import traceback
from sys import exit
from time import sleep
from subprocess import Popen, PIPE
from datetime import datetime

"""
Developer: Birchy (support@bespokebots.com)
Date: 20/01/2013
Purpose: Wifi drivers increase latency, thus reducing LnuxCNC performance. This script
    monitors the running processes and disables the wifi driver if either 'axis' or 
    'pyvcp' (latency test) are running. If neither of these processes are running
    AND the wifi driver is disabled, the script will re-enable the wifi driver.
    Network manager can be left running and will auto-magically disconnect/re-connect
    depending on driver status.
Useage: This script MUST be run as ROOT in order to use modprobe for enabling/disabling
    the wifi driver. Assuming that this script is currently located in your HOME folder,
    make it start automatically at reboot with root permission using these commands:
    "sudo cp wifi_daemon.py /usr/bin" (copy this file into /usr/bin)
    "sudo crontab -e" (launch crontab editor for root)
    add this line to the bottom of crontab:
        "@reboot /usr/bin/python /usr/bin/wifi_daemon.py"
    Save and exit crontab editor.
    Reboot your computer.
    This script will then auto-start and run as a background process, automatically
    managing the loading/unloading of the wifi driver.
    You will need to edit the 'mod_name' string below to match the name of YOUR wifi
    driver. To find the name of your wifi driver, run these commands:
        "dmesg | grep wlan0"
        ...then look for a line like 'using XXX driver'
        ...mine says 'using NDIS driver'
        ...so now we can run:
        "lsmod | grep -i NDIS"
        ...which on my computer shows that the current wifi driver is 'ndiswrapper'
"""

ERR_LOG_PATH = '/var/log/wifi_daemon.ERROR'
blacklist = ['/usr/bin/axis', '/usr/bin/pyvcp'] # processes = axis & latency test
mod_name = 'ndiswrapper' # wifi driver name. *** See notes above! ***

while 1:
    try:
        # get running processes & modules
        procs = Popen(['ps', '-A', 'x'], shell = False, stdout = PIPE, stderr = PIPE).communicate()[0]
        mods = Popen(['lsmod'], shell = False, stdout = PIPE, stderr = PIPE).communicate()[0]

        # check if a blacklisted process is running
        blacklisted_running = False
        for proc in blacklist:
            if proc in procs:
                blacklisted_running = True
                break

        # check if driver module is loaded
        module_loaded = False
        if mod_name in mods: module_loaded = True

        # decide if we should load or unload the driver module
        if blacklisted_running and module_loaded:
            # UNLOAD the driver module...
            ret = Popen(['/sbin/modprobe', '-r', mod_name], shell = False, stdout = PIPE, stderr = PIPE).communicate()[0]
        elif not blacklisted_running and not module_loaded:
            # RELOAD the driver module...
            ret = Popen(['/sbin/modprobe', mod_name], shell = False, stdout = PIPE, stderr = PIPE).communicate()[0]
    except:
        # log errors to file and exit
	    now = datetime.now().strftime('%d/%m/%Y %H:%M:%S')
	    err = traceback.format_exc()
	    s = '%s\n%s\n\n' % (now, err)
	    open(ERR_LOG_PATH, 'w').write(s) # 'w' prevents file getting large
	    exit()
    # prevent cpu hog
    sleep(0.5)

